SystemOrganization addCategory: #'MCConverter-Incremental'!

!Fraction methodsFor: '*mcconverter-arithmetic' stamp: '<nobody> 4/4/2005 01:29'!
inc: aFraction
	^ Fraction
		numerator: (numerator * denominator * aFraction denominator) 
						+ (aFraction numerator * denominator)
		denominator: (denominator * denominator * aFraction denominator)! !

Object subclass: #MDIncrementalConverter
	instanceVariableNames: 'source destination workspace proxy reparents parents progress elements includeRemovals'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MCConverter-Incremental'!

!MDIncrementalConverter class methodsFor: 'as yet unclassified' stamp: 'cwp 5/18/2008 11:16'!
convertMC2
	| converter |
	converter := self from: 'mc'.
	converter ignoreRemovals.
	converter make: 'MC2-Repositories-cwp.1' descendFrom: #('Monticello2-cwp.206').
	converter convertVersionNamed: 'MCConverter-cwp.25'.
	converter convertVersionNamed: 'MC2-Extras-cwp.3'.
	converter convertVersionNamed: 'Monticello2-cwp.248'.
	converter convertVersionNamed: 'MC2-Repositories-cwp.19'.
	converter convertVersionNamed: 'MC2-Squeak-cwp.46'.
	converter convertVersionNamed: 'MC2-SqueakPlatform-cwp.24'.
	converter convertVersionNamed: 'MC2-SqueakUI-cwp.52'.
	converter saveTo: 'mc-converted'.
	converter explore

! !

!MDIncrementalConverter class methodsFor: 'as yet unclassified' stamp: '<nobody> 5/5/2008 07:37'!
from: aString
	^ self from: aString to: aString! !

!MDIncrementalConverter class methodsFor: 'as yet unclassified' stamp: '<nobody> 5/16/2008 07:57'!
from: sourceName to: destName
	| source destination |
	source := MCDirectoryRepository new.
	source directory: (FileDirectory default directoryNamed: sourceName).
	destination := MDMemoryRepository new.
	^ self source: source destination: destination! !

!MDIncrementalConverter class methodsFor: 'as yet unclassified' stamp: '<nobody> 5/5/2008 07:39'!
source: mc1Repo destination: mc2Repo
	^ self new setSource: mc1Repo destination: mc2Repo! !

!MDIncrementalConverter methodsFor: 'initialize-release' stamp: '<nobody> 5/16/2008 20:55'!
ancestorsOf: aVersionInfo from: aCollection
	^ self ancestorsOf: aVersionInfo satisfying: [:ea | aCollection includes: ea name]! !

!MDIncrementalConverter methodsFor: 'converting' stamp: '<nobody> 5/16/2008 08:11'!
ancestorsOf: aVersionInfo satisfying: aBlock
	| ancestors |
	ancestors := reparents 
		at: aVersionInfo id
		ifAbsent: [ aVersionInfo ancestors ].
	^ ancestors 
		inject: OrderedCollection new
		into:
			[:all :each |
			(aBlock value: each)
				ifTrue: [all add: each]
				ifFalse: [all addAll: (self ancestorsOf: each satisfying: aBlock)].
			all]! !

!MDIncrementalConverter methodsFor: 'converting' stamp: '<nobody> 5/16/2008 08:18'!
convertQueue: queue
	queue do: [:ea | self convertVersionInfo: ea]! !

!MDIncrementalConverter methodsFor: 'converting' stamp: '<nobody> 5/16/2008 08:00'!
convertVersion: aVersion 
	| memo |
	self loadAncestorsOf: aVersion info.
	self installVersion: aVersion.
	memo := self saveSnapshotFromVersion: aVersion.
	progress advance.
	^ memo! !

!MDIncrementalConverter methodsFor: 'converting' stamp: '<nobody> 5/16/2008 08:18'!
convertVersionInfo: aVersionInfo
	| version |
	version := source 
		versionWithInfo: aVersionInfo
		ifAbsent: [nil].
	^ version 
		ifNil:[self error: 'version ', aVersionInfo name, ' not found']
		ifNotNil: [self convertVersion: version]! !

!MDIncrementalConverter methodsFor: 'public interface' stamp: '<nobody> 5/16/2008 23:24'!
convertVersionNamed: aString 
	| version queue |
	version := source versionFromFileNamed: aString, '.mcz'.
	queue := self queueForVersionInfo: version info.
	queue isEmpty ifFalse:
		[self initializeProgressForQueue: queue.
		self installPlatformDuring: [self convertQueue: queue].
		progress pop]! !

!MDIncrementalConverter methodsFor: 'initialize-release' stamp: '<nobody> 5/11/2008 17:51'!
createWorkspace
	proxy := MDFakeImageProxy new.
	workspace := MDWorkingCopy forImageProxy: proxy! !

!MDIncrementalConverter methodsFor: 'accessing' stamp: '<nobody> 5/11/2008 00:43'!
destination
	^ destination! !

!MDIncrementalConverter methodsFor: 'accessing' stamp: '<nobody> 5/11/2008 00:43'!
destination: anObject
	destination := anObject! !

!MDIncrementalConverter methodsFor: 'writing' stamp: 'cwp 5/18/2008 20:12'!
elements
	^ includeRemovals
		ifTrue: [proxy allElements]
		ifFalse: [proxy elements]! !

!MDIncrementalConverter methodsFor: 'loading' stamp: '<nobody> 5/18/2008 00:42'!
expandVersions: aSet 
	| existingElements result |
	^ includeRemovals 
		ifFalse: [aSet]
		ifTrue: 
			[result := aSet copy.
			existingElements := aSet collect: [ :ea | ea element ].
			proxy elements do: 
				[ :el | 
				(existingElements includes: el) ifFalse: 
					[result add: (MDNullVersion forElement: el)]].
			result]! !

!MDIncrementalConverter methodsFor: 'accessing' stamp: '<nobody> 5/18/2008 00:40'!
ignoreRemovals
	includeRemovals := false! !

!MDIncrementalConverter methodsFor: 'accessing' stamp: '<nobody> 5/18/2008 00:39'!
includeRemovals
	includeRemovals := true! !

!MDIncrementalConverter methodsFor: 'initialize-release' stamp: '<nobody> 5/16/2008 21:01'!
initializeProgressForQueue: queue

	progress := MDNestedTask withLabel: 'Converting ', queue last name.
	progress push: queue size! !

!MDIncrementalConverter methodsFor: 'installing' stamp: '<nobody> 5/11/2008 23:02'!
installClassComment: aDefinition 
	| element variant |
	element := MDClassCommentElement className: aDefinition className.
	variant := MDDefinitionVariant forElement: element.
	variant properties
		text: aDefinition comment;
		initials: (aDefinition commentStamp copyUpTo: $ );
		timeStamp: (MDTimestamp fromSqueakMethodStamp: aDefinition commentStamp).
	variant loadInto: proxy.
	elements add: element! !

!MDIncrementalConverter methodsFor: 'installing' stamp: '<nobody> 5/11/2008 23:02'!
installClassDefinition: aDefinition 
	| element variant |
	element := MDClassElement className: aDefinition className.
	variant := MDDefinitionVariant forElement: element.
	variant properties
		category: aDefinition category;
		format: aDefinition type;
		superclassName: aDefinition superclassName.
	variant loadInto: proxy.
	aDefinition hasComment ifTrue: [ self installClassComment: aDefinition ].
	self installVariablesFromClass: aDefinition.
	elements add: element! !

!MDIncrementalConverter methodsFor: 'installing' stamp: '<nobody> 5/8/2008 22:14'!
installDefinition: aDefinition 
	aDefinition isClassDefinition ifTrue: [ ^ self installClassDefinition: aDefinition ].
	aDefinition isMethodDefinition ifTrue: [ ^ self installMethodDefinition: aDefinition ]! !

!MDIncrementalConverter methodsFor: 'installing' stamp: 'cwp 5/18/2008 11:15'!
installMethodDefinition: aDefinition 
	| element variant |
	element := MDMethodElement 
		className: aDefinition className
		selector: aDefinition selector
		classIsMeta: aDefinition classIsMeta.
	variant := MDDefinitionVariant forElement: element.
	variant properties
		initials: (aDefinition timeStamp copyUpTo: $ );
		category: aDefinition category;
		source: aDefinition source;
		timeStamp: (aDefinition timeStamp isEmpty ifFalse: [ MDTimestamp fromMC1Stamp: aDefinition timeStamp ]).
	variant loadInto: proxy.
	elements add: element! !

!MDIncrementalConverter methodsFor: 'writing' stamp: '<nobody> 5/10/2008 18:55'!
installPlatformDuring: aBlock
	| platform |
	platform := MDPlatform current.
	[MDPlatform current: MDConversionPlatform new.
	aBlock value]
		ensure: [MDPlatform current: platform]! !

!MDIncrementalConverter methodsFor: 'installing' stamp: '<nobody> 5/11/2008 23:03'!
installVariablesFromClass: aClassDefinition 
	| class element |
	aClassDefinition variables do: 
		[ :variable | 
		variable isInstanceVariable ifTrue: [ class := MDInstVarElement ].
		variable isClassVariable ifTrue: [ class := MDClassVarElement ].
		variable isClassInstanceVariable ifTrue: [ class := MDClassInstVarElement ].
		variable isPoolImport ifTrue: [ class := MDPoolImportElement ].
		element := class 
			className: aClassDefinition className
			name: variable name.
		(MDDefinitionVariant forElement: element) loadInto: proxy.
		elements add: element ]! !

!MDIncrementalConverter methodsFor: 'installing' stamp: '<nobody> 5/17/2008 10:51'!
installVersion: mc1Version 
	elements := Set new.
	mc1Version snapshot definitions do: [ :ea | self installDefinition: ea ].
	(proxy elements difference: elements) do: [:ea | proxy removeElement: ea].
	elements := nil! !

!MDIncrementalConverter methodsFor: 'loading' stamp: '<nobody> 5/17/2008 22:59'!
loadAncestorsOf: aVersionInfo
	| memos all ancestors |
	self createWorkspace.
	all := destination memos.
	ancestors := self ancestorsOf: aVersionInfo satisfying: 
		[:info | all anySatisfy: 
			[:memo | info id asString = memo properties uuid]].
	memos := ancestors collect: [:info | self memoWithUuid: info id].
	memos isEmpty ifFalse:
		[self loadMemo: memos first.
		memos allButFirstDo: [:ea | self mergeMemo: ea]]! !

!MDIncrementalConverter methodsFor: 'loading' stamp: '<nobody> 5/11/2008 17:57'!
loadMemo: memo 
	| snapshot |
	parents := Array with: memo.
	snapshot := destination get: memo reference.
	^ self loadSnapshot: snapshot! !

!MDIncrementalConverter methodsFor: 'loading' stamp: '<nobody> 5/17/2008 22:58'!
loadSnapshot: aSnapshot 
	| versions expanded |
	versions := aSnapshot 
		getVersionsFrom: destination
		for: workspace.
	expanded := self expandVersions: versions.
	workspace transaction: [ expanded do: [ :ea | workspace loadVersion: ea ] ]! !

!MDIncrementalConverter methodsFor: 'public interface' stamp: '<nobody> 5/12/2008 22:13'!
make: childString descendFrom: parentArray
	| child newParents |
	child := self versionNamed: childString.
	newParents := parentArray collect: [:ea | (self versionNamed: ea) info].
	reparents at: child info id put: newParents! !

!MDIncrementalConverter methodsFor: 'reading' stamp: '<nobody> 5/15/2008 23:00'!
memoWithUuid: id
	^ destination memos detect: [:ea | ea properties uuid = id asString] 
	! !

!MDIncrementalConverter methodsFor: 'loading' stamp: '<nobody> 5/11/2008 17:54'!
mergeMemo: memo 
	| versions snapshot |
	snapshot := destination get: memo reference.
	versions := snapshot 
		getVersionsFrom: destination
		for: workspace.
	versions := self expandVersions: versions.
	versions do: 
		[ :ea | 
		workspace 
			includeVersion: ea
			onConflict: [ :l :r | l variant ] ].
	^ memo! !

!MDIncrementalConverter methodsFor: 'converting' stamp: '<nobody> 5/10/2008 19:11'!
ordinalFromVersionName: aString
	^ (aString copyAfterLast: $.) asNumber! !

!MDIncrementalConverter methodsFor: 'writing' stamp: '<nobody> 5/11/2008 01:37'!
parentReferences
	^ (parents ifNil: [ Array new ]) collect: [ :ea | ea hashstamp ]! !

!MDIncrementalConverter methodsFor: 'initialize-release' stamp: '<nobody> 5/15/2008 22:09'!
push: aVersionInfo onQueue: queue memos: memos versions: versions
	| id |
	parents := (self ancestorsOf: aVersionInfo from: versions) 
				reject: [:ea | memos includes: ea id asString].
	parents do: [:ea | self push: ea onQueue: queue memos: memos versions: versions].
	id := aVersionInfo id asString.
	(memos includes: id) ifFalse: 
		[memos add: aVersionInfo id asString.
		queue add: aVersionInfo]! !

!MDIncrementalConverter methodsFor: 'initialize-release' stamp: '<nobody> 5/16/2008 08:16'!
queueForVersionInfo: aVersionInfo
	| memos queue versions |
	memos := destination memos asOrderedCollection collect: [:ea | ea properties uuid].
	versions := source readableFileNames collect: [:ea | ea allButLast: 4].
	queue := OrderedCollection new.
	self push: aVersionInfo onQueue: queue memos: memos versions: versions.
	^ queue! !

!MDIncrementalConverter methodsFor: 'writing' stamp: 'cwp 5/18/2008 20:12'!
saveMemo: memo withSlice: slice 
	| objects snapshot |
	objects := Set new.
	self elements do: [ :ea | objects add: (workspace versionForElement: ea) ].
	snapshot := MDSnapshot withVersions: objects.
	objects add: snapshot.
	memo 
		setSnapshot: snapshot
		slice: slice.
	objects add: memo.
	destination putAll: objects! !

!MDIncrementalConverter methodsFor: 'writing' stamp: '<nobody> 5/11/2008 01:37'!
saveSnapshotFromVersion: aVersion 
	| properties memo slice |
	self setTimestampFromInfo: aVersion info.
	properties := MDPropertyDictionary new
		uuid: aVersion info id asString;
		initials: aVersion info author;
		ordinal: (self ordinalFromVersionName: aVersion name);
		comment: aVersion info message;
		sliceName: aVersion package name;
		yourself.
	memo := MDMemo 
		parents: self parentReferences
		properties: properties.
	slice := MDPackageInfoSlice named: aVersion package name.
	self 
		saveMemo: memo
		withSlice: slice.
	^ memo! !

!MDIncrementalConverter methodsFor: 'writing' stamp: '<nobody> 5/16/2008 23:31'!
saveTo: aString
	| file |
	file := MDFileRepository named: aString.
	[destination pushTo: file]
		ensure: [file close]! !

!MDIncrementalConverter methodsFor: 'initialize-release' stamp: '<nobody> 5/18/2008 00:39'!
setSource: mc1Repo destination: mc2Repo 
	source := mc1Repo.
	destination := mc2Repo.
	reparents := Dictionary new.
	self includeRemovals.
	self createWorkspace.
	! !

!MDIncrementalConverter methodsFor: 'writing' stamp: '<nobody> 5/10/2008 18:47'!
setTimestampFromInfo: aVersionInfo 
	MDPlatform current 
		setDate: aVersionInfo date
		time: aVersionInfo time! !

!MDIncrementalConverter methodsFor: 'accessing' stamp: '<nobody> 5/11/2008 00:43'!
source
	^ source! !

!MDIncrementalConverter methodsFor: 'accessing' stamp: '<nobody> 5/11/2008 00:43'!
source: anObject
	source := anObject! !

!MDIncrementalConverter methodsFor: 'reading' stamp: '<nobody> 5/14/2008 07:11'!
versionNamed: aString
	| filename |
	filename := aString, '.mcz'.
	^ source loadVersionFromFileNamed: filename! !

Object subclass: #MDNestedTask
	instanceVariableNames: 'stack update done'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MCConverter-Incremental'!

!MDNestedTask class methodsFor: 'as yet unclassified' stamp: '<nobody> 3/21/2008 14:06'!
withLabel: aString
	| morph |
	morph _ ProgressMorph label: aString.
	morph openInWorld.
	^ self basicNew
		setUpdate: [:v | morph done: v]
		done: [morph delete]! !

!MDNestedTask methodsFor: 'public' stamp: '<nobody> 4/4/2005 01:21'!
advance
	
	| top |
	top _ stack last.
	top _ Fraction
			numerator: top numerator + 1
			denominator: top denominator.
	stack at: stack size put: top.
	self update! !

!MDNestedTask methodsFor: 'public' stamp: '<nobody> 3/21/2008 13:21'!
pop
	stack removeLast.
	stack isEmpty ifTrue: [done value]! !

!MDNestedTask methodsFor: 'public' stamp: '<nobody> 4/4/2005 01:08'!
push: aNumber
	stack add: (Fraction numerator: 0 denominator: aNumber)! !

!MDNestedTask methodsFor: 'initialize-release' stamp: '<nobody> 3/21/2008 13:20'!
setUpdate: updateBlock done: doneBlock
	stack _ OrderedCollection new.
	update _ updateBlock.
	done _ doneBlock.! !

!MDNestedTask methodsFor: 'public' stamp: '<nobody> 3/21/2008 13:21'!
update
	update value: (stack inject: 0 asFraction into: [:sum :ea | sum inc: ea])! !

MDPlatform subclass: #MDConversionPlatform
	instanceVariableNames: 'seconds'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MCConverter-Incremental'!

!MDConversionPlatform methodsFor: 'as yet unclassified' stamp: '<nobody> 5/10/2008 18:44'!
setDate: date time: time
	| ts |
	ts := MDTimestamp
		year: date year
		month: date monthIndex
		day: date dayOfMonth
		hour: time hour
		minute: time minute
		second: time second.
	seconds := ts asInteger! !

!MDConversionPlatform methodsFor: 'as yet unclassified' stamp: '<nobody> 5/15/2008 07:21'!
timestampSeconds
	^ seconds ifNil: [super timestampSeconds]! !

!MDTimestamp class methodsFor: '*mcconverter' stamp: '<nobody> 4/23/2008 08:01'!
fromMC1Stamp: aString
	| stream month day year hour minute tmp |
	stream := ReadStream on: aString.
	[stream peek isDigit] whileFalse: [stream next].
	month := (stream upTo: $/) asNumber.
	day := (stream upTo: $/) asNumber.
	year := (stream upTo: Character space) asNumber.
	hour := (stream upTo: $:) asNumber.
	minute := (stream upTo: Character space) asNumber.
	stream atEnd ifFalse: [(stream next: 2) = 'pm' ifTrue: [hour := hour + 12]].
	
	"Apparently this is possible, though rare"
	year < 1000 ifTrue: 
		[tmp := month.
		month := day.
		day := month.
		year := year + (year < 80 ifTrue: [2000] ifFalse: [1900])].
	
	^ self year: year month: month day: day hour: hour minute: minute second: 0! !
